/*
	Arduino library for TruStability pressure sensors
	SPI Models supported.
	AdAstra Team - May 2019
*/

#include "Arduino.h"
#include "Honeywell_pressure_sensors.h"
#include <SPI.h>

HPS::HPS(int pin) {
    pinMode(pin, OUTPUT);
    digitalWrite(pin, HIGH);
    _pin = pin;
}

float HPS::readPressure() {
    int whichChip = _pin;

    byte firstByte;
    byte secondByte;

    // asserting this sensor by bringing CS pin low
    digitalWrite(whichChip, LOW);
    // here we're asking for our two bytes
    firstByte = SPI.transfer(0x00);
    secondByte = SPI.transfer(0x00);
    // de-asserting this sensor by bringing CS pin high
    digitalWrite(whichChip, HIGH);

    // here we're bitshifting the first byte 8 bits to the left
    uint16_t bothBytes = (firstByte << 8) | secondByte;
    float pressure = pressuretransfer(bothBytes);
    return pressure;
}

float HPS::readTemperature(){
    int whichChip = _pin;

    byte firstByte;
    byte secondByte;
    byte thirdByte;
    byte fourthByte;

    // asserting this sensor by bringing CS pin low
    digitalWrite(whichChip, LOW);
    // here we're asking for our two bytes
    firstByte = SPI.transfer(0x00);
    secondByte = SPI.transfer(0x00);
    thirdByte = SPI.transfer(0x00);
    fourthByte = SPI.transfer(0x00);
    // de-asserting this sensor by bringing CS pin high
    digitalWrite(whichChip, HIGH);

    // here we're bitshifting the third byte 8 bits to the left
    uint16_t bothBytes = (thirdByte << 3);// >> 3);// << 8);// | (fourthByte >> 5);
    //int bothBytes = thirdByte;
    float temperature = temperaturetransfer(bothBytes);

    // de-asserting this sensor by bringing CS pin high
    digitalWrite(whichChip, HIGH);

    return temperature;
}

int HPS::getStatus(){
    int whichChip = _pin;
    int status;
    bool S0;
    bool S1;
    byte firstByte;
    // asserting this sensor by bringing CS pin low
    digitalWrite(whichChip, LOW);
    // Reading First Byte
    firstByte = SPI.transfer(0x00);
    // de-asserting this sensor by bringing CS pin high
    digitalWrite(whichChip, HIGH);
    status = (firstByte >> 6);
    return status;
    /*if (status == 0)
	    return "Normal operation, valid data"
    elsif (status == 1) 
	    return "Device in command mode, should not be seen"
    elsif (status == 2)
	    return "Stale data: data already fetched"	
    else
	    return "Diagnostic condition"
	*/
}

float HPS::pressuretransfer(uint16_t dataIn) {
    float outputMax = 14745.0; // 90 percent of 2 to the 14th power (from sensor's data sheet)
    float outputMin = 1638.0; // 10 percent of 2 to the 14th power (from sensor's data sheet)
    float pressureMax = 60.0; // max 60 psi (from sensor's datea sheet)
    float pressureMin = -60.0;
    float pressure = 0.0;

    // transfer function: using sensor output to solve for pressure
    pressure = (dataIn - outputMin) * (pressureMax - pressureMin) / (outputMax - outputMin);
    pressure = pressure + pressureMin;
    //Serial.print(pressure);
    return (pressure);
}

float HPS::temperaturetransfer(uint16_t dataIn) {
    // transfer function: using sensor output to solve for temperature
    float temperature = ((dataIn/2047.)*200.)-50.;
    return(temperature);
}

